@@ -30,26 +30,23 @@ Sjors H.W. Scheres, Takanori Nakane, Colin M. Palmer, Donovan Webb"""
 import argparse
 import json
 import os
-import shutil
 import time
 from glob import glob
 import subprocess
 import math
-from emtable import Table  # requires pip install emtable
+from emtable import Table  # run "pip3 install --user emtable" for system python
 
 
-RELION_JOB_FAILURE_FILENAME = "RELION_JOB_EXIT_FAILURE"
-RELION_JOB_SUCCESS_FILENAME = "RELION_JOB_EXIT_SUCCESS"
-DONE_MICS = "done_mics.txt"
-CONDA_ENV = ". /home/gsharov/rc/conda.rc && conda activate cryolo-1.7.7"
+CONDA_ENV = ". /home/gsharov/rc/conda.rc && conda activate cryolo-1.8.2"
 CRYOLO_PREDICT = "cryolo_predict.py"
 CRYOLO_GEN_MODEL = "/home/gsharov/soft/cryolo/gmodel_phosnet_202005_N63_c17.h5"
 CRYOLO_GEN_JANNI_MODEL = "/home/gsharov/soft/cryolo/gmodel_phosnet_202005_nn_N63_c17.h5"
 CRYOLO_JANNI_MODEL = "/home/gsharov/soft/cryolo/gmodel_janni_20190703.h5"
+SCRATCH_DIR = os.getenv("RELION_SCRATCH_DIR", None)  # SSD scratch space for filtered mics, can be None
 DEBUG = 0
 
 
-def run_job(project_dir, args):
+def run_job(args):
     start = time.time()
     in_mics = args.in_mics
     job_dir = args.out_dir
@@ -59,31 +56,37 @@ def run_job(project_dir, args):
     model = args.model
     filament = args.filament
     if filament:
-        fil_width = args.filament_width
         box_dist = args.box_distance
         min_boxes = args.minimum_number_boxes
     denoise = args.denoise
     gpus = args.gpu
     threads = args.threads
 
-    getPath = lambda *arglist: os.path.join(project_dir, *arglist)
+    if SCRATCH_DIR is not None:
+        filtered_dir = os.path.join(SCRATCH_DIR, "filtered_tmp")
+    else:
+        filtered_dir = "%s/filtered_tmp/" % job_dir
 
     if model == "None":
         model = CRYOLO_GEN_MODEL if not denoise else CRYOLO_GEN_JANNI_MODEL
     else:
-        model = getPath(model)
+        model = os.path.abspath(model)
 
     # Making a cryolo config file
-    json_dict = {"model": {
-        "architecture": "PhosaurusNet",
-        "input_size": 1024,
-        "max_box_per_image": 600,
-        "filter": [
-            0.1,
-            "filtered_tmp/"
-        ]
-    }}
-
+    json_dict = {
+        "model": {
+            "architecture": "PhosaurusNet",
+            "input_size": 1024,
+            "max_box_per_image": 600,
+            "filter": [
+                0.1,
+                filtered_dir
+            ]
+        },
+        "other": {
+            "log_path": "%s/logs/" % job_dir
+        }
+    }
     if box_size:  # is not 0
         json_dict["model"]["anchors"] = [int(box_size), int(box_size)]
         if not filament:
@@ -93,86 +96,45 @@ def run_job(project_dir, args):
             CRYOLO_JANNI_MODEL,
             24,
             3,
-            "filtered_tmp/"
+            filtered_dir
         ]
 
     if DEBUG:
         print("Using following config: ", json_dict)
 
-    with open("config_cryolo.json", "w") as json_file:
+    with open(os.path.join(job_dir, "config_cryolo.json"), "w") as json_file:
         json.dump(json_dict, json_file, indent=4)
 
-    # Reading the micrographs star file from relion
-    mictable = Table(fileName=getPath(in_mics), tableName='micrographs')
-
-    # Arranging files for cryolo: making symlinks for mics
-    done_mics = []
-    mic_dirs = []
-    if os.path.exists(DONE_MICS):
-        with open(DONE_MICS, "r") as f:
-            done_mics.extend(f.read().splitlines())
-    if DEBUG:
-        print("Current done_mics: ", done_mics)
-
+    # Reading the micrographs star file from Relion
+    mictable = Table(fileName=in_mics, tableName='micrographs')
     mic_fns = mictable.getColumnValues("rlnMicrographName")
-    mic_ext = os.path.splitext(mic_fns[0])[1]
-    if "job" in mic_fns[0].split("/")[1]:  # remove JobType/jobXXX
-        input_job = "/".join(mic_fns[0].split("/")[:2])
-        keys = ["/".join(i.split("/")[2:]) for i in mic_fns]
-    else:
-        input_job = ""
-        keys = mic_fns
-    values = [os.path.splitext(i)[0] + "_autopick.star" for i in keys]
-    mic_dict = {k: v for k, v in zip(keys, values) if k not in done_mics}
-
-    for mic in mic_dict:
-        if DEBUG:
-            print("Processing mic: ", mic)
-        mic_dir = os.path.dirname(mic)
-        # create folder for micrograph links for cryolo job
-        if not os.path.isdir(mic_dir):
-            os.makedirs(mic_dir)
-        if mic_dir not in mic_dirs:
-            mic_dirs.append(mic_dir)
-            if DEBUG:
-                print("Added folder %s to the mic_dirs" % mic_dir)
-        inputfn = getPath(input_job, mic)
-        outfn = getPath(job_dir, mic)
-        os.symlink(inputfn, outfn)
-        if DEBUG:
-            print("Link %s --> %s" % (inputfn, outfn))
-
-    if len(mic_dict.keys()) == 0:
-        print("All mics picked! Nothing to do.")
-        open(RELION_JOB_SUCCESS_FILENAME, "w").close()
-        exit(0)
 
     # Launching cryolo
     args_dict = {
-        '--conf': "config_cryolo.json",
-        '--output': 'output',
+        '--conf': os.path.join(job_dir, "config_cryolo.json"),
+        '--input': in_mics,
+        '--output': os.path.join(job_dir, 'output'),
         '--weights': model,
         '--gpu': gpus.replace(',', ' '),
         '--threshold': thresh,
         '--distance': distance,
         '--cleanup': "",
+        '--skip': "",
+        '--write_empty': "",
         '--num_cpu': -1 if threads == 1 else threads
     }
 
     if filament:
         args_dict.update({
             '--filament': "",
-            '--filament_width': fil_width,
             '--box_distance': box_dist,
-            '--minimum_number_boxes': min_boxes
+            '--minimum_number_boxes': min_boxes,
+            '--directional_method': 'PREDICTED'
         })
+        args_dict.pop('--distance')
 
     cmd = "%s && %s " % (CONDA_ENV, CRYOLO_PREDICT)
     cmd += " ".join(['%s %s' % (k, v) for k, v in args_dict.items()])
-    cmd += " --input "
-    for i in mic_dirs:
-        if len(glob("%s/*%s" % (i, mic_ext))):  # skip folders with no mics
-            cmd += "%s/ " % i
 
     print("Running command:\n{}".format(cmd))
     proc = subprocess.Popen(cmd, shell=True)
@@ -183,31 +145,36 @@ def run_job(project_dir, args):
 
     # Moving output star files for Relion to use
     table_coords = Table(columns=['rlnMicrographName', 'rlnMicrographCoordinates'])
-    with open(DONE_MICS, "a+") as f, open("autopick.star", "w") as mics_star:
-        for mic in mic_dict:
-            f.write("%s\n" % mic)
+    star_dir = "EMAN_HELIX_SEGMENTED" if filament else "STAR"
+    ext = ".box" if filament else ".star"
+    with open(os.path.join(job_dir, "autopick.star"), "w") as mics_star:
+        for mic in mic_fns:
             mic_base = os.path.basename(mic)
-            os.remove(mic)  # clean up symlink
-            coord_cryolo = os.path.splitext(mic_base)[0] + ".star"
-            coord_cryolo = getPath(job_dir, "output", "STAR", coord_cryolo)
-            coord_relion = mic_dict[mic]
+            mic_dir = os.path.dirname(mic)
+            if len(mic_dir.split("/")) > 1 and "job" in mic_dir.split("/")[1]:  # remove JobType/jobXXX
+                mic_dir = "/".join(mic_dir.split("/")[2:])
+            os.makedirs(os.path.join(job_dir, mic_dir), exist_ok=True)
+            coord_cryolo = os.path.splitext(mic_base)[0] + ext
+            coord_cryolo = os.path.join(job_dir, "output", star_dir, coord_cryolo)
+            coord_relion = os.path.splitext(mic_base)[0] + "_autopick" + ext
+            coord_relion = os.path.join(job_dir, mic_dir, coord_relion)
             if os.path.exists(coord_cryolo):
-                os.rename(coord_cryolo, getPath(job_dir, coord_relion))
-                table_coords.addRow(os.path.join(input_job, mic), os.path.join(job_dir, coord_relion))
+                os.rename(coord_cryolo, coord_relion)
+                table_coords.addRow(mic, coord_relion)
                 if DEBUG:
-                    print("Moved %s to %s" % (coord_cryolo, getPath(job_dir, coord_relion)))
+                    print("Moved %s to %s" % (coord_cryolo, coord_relion))
         table_coords.writeStar(mics_star, tableName='coordinate_files')
 
-    # Required output job_pipeline.star file
-    pipeline_fn = getPath(job_dir, "job_pipeline.star")
+    # Required output to mini pipeline job_pipeline.star file
+    pipeline_fn = os.path.join(job_dir, "job_pipeline.star")
     table_gen = Table(columns=['rlnPipeLineJobCounter'])
     table_gen.addRow(2)
     table_proc = Table(columns=['rlnPipeLineProcessName', 'rlnPipeLineProcessAlias',
                                 'rlnPipeLineProcessTypeLabel', 'rlnPipeLineProcessStatusLabel'])
     table_proc.addRow(job_dir, 'None', 'relion.external', 'Running')
     table_nodes = Table(columns=['rlnPipeLineNodeName', 'rlnPipeLineNodeTypeLabel'])
-    table_nodes.addRow(in_mics, "relion.MicrographStar")
-    table_nodes.addRow(os.path.join(job_dir, "autopick.star"), "relion.CoordinateStar")
+    table_nodes.addRow(in_mics, "MicrographsData.star.relion")
+    table_nodes.addRow(os.path.join(job_dir, "autopick.star"), "MicrographsCoords.star.relion.autopick")
     table_input = Table(columns=['rlnPipeLineEdgeFromNode', 'rlnPipeLineEdgeProcess'])
     table_input.addRow(in_mics, job_dir)
     table_output = Table(columns=['rlnPipeLineEdgeProcess', 'rlnPipeLineEdgeToNode'])
@@ -220,11 +187,14 @@ def run_job(project_dir, args):
         table_input.writeStar(f, tableName="pipeline_input_edges")
         table_output.writeStar(f, tableName="pipeline_output_edges")
 
-    outputFn = getPath(job_dir, "output_for_relion.star")
+    # Register output nodes in .Nodes/
+    os.makedirs(os.path.join(".Nodes", "MicrographsCoords", job_dir), exist_ok=True)
+    open(os.path.join(".Nodes", "MicrographsCoords", job_dir, "autopick.star"), "w").close()
+
+    outputFn = os.path.join(job_dir, "output_for_relion.star")
     if not os.path.exists(outputFn):
         # get estimated box size
-        summaryfn = getPath(job_dir, "output/DISTR",
-                            'size_distribution_summary*.txt')
+        summaryfn = os.path.join(job_dir, "output/DISTR", 'size_distribution_summary*.txt')
         with open(glob(summaryfn)[0]) as f:
             for line in f:
                 if line.startswith("MEAN,"):
@@ -233,50 +203,61 @@ def run_job(project_dir, args):
         print("\ncrYOLO estimated box size %d px" % estim_sizepx)
 
         # calculate diameter, original (boxSize) and downsampled (boxSizeSmall) box
-        optics = Table(fileName=getPath(in_mics), tableName='optics')
+        optics = Table(fileName=in_mics, tableName='optics')
         angpix = float(optics[0].rlnMicrographPixelSize)
-        # use + 20% for diameter
-        diam = math.ceil(estim_sizepx * angpix * 1.2)
-        # use +30% for box size, make it even
-        boxSize = 1.3 * estim_sizepx
-        boxSize = math.ceil(boxSize / 2.) * 2
 
-        # from relion_it.py script
-        # Authors: Sjors H.W. Scheres, Takanori Nakane & Colin M. Palmer
-        boxSizeSmall = None
-        for box in (48, 64, 96, 128, 160, 192, 256, 288, 300, 320,
-                    360, 384, 400, 420, 450, 480, 512, 640, 768,
-                    896, 1024):
-            # Don't go larger than the original box
-            if box > boxSize:
-                boxSizeSmall = boxSize
-                break
-            # If Nyquist freq. is better than 8.5 A, use this
-            # downscaled box, otherwise continue to next size up
-            small_box_angpix = angpix * boxSize / box
-            if small_box_angpix < 4.25:
-                boxSizeSmall = box
-                break
+        if filament:
+            # box size = 1.5x tube diam
+            diam = 0.66 * box_size
+        else:
+            # use + 20% for diameter
+            diam = math.ceil(estim_sizepx * angpix * 1.2)
+            # use +30% for box size, make it even
+            boxSize = 1.3 * estim_sizepx
+            boxSize = math.ceil(boxSize / 2.) * 2
 
-        print("\nSuggested parameters:\n\tDiameter (A): %d\n\tBox size (px): %d\n"
-              "\tBox size binned (px): %d" % (diam, boxSize, boxSizeSmall))
+            # from relion_it.py script
+            # Authors: Sjors H.W. Scheres, Takanori Nakane & Colin M. Palmer
+            boxSizeSmall = None
+            for box in (48, 64, 96, 128, 160, 192, 256, 288, 300, 320,
+                        360, 384, 400, 420, 450, 480, 512, 640, 768,
+                        896, 1024):
+                # Don't go larger than the original box
+                if box > boxSize:
+                    boxSizeSmall = boxSize
+                    break
+                # If Nyquist freq. is better than 7.5 A, use this
+                # downscaled box, otherwise continue to next size up
+                small_box_angpix = angpix * boxSize / box
+                if small_box_angpix < 3.75:
+                    boxSizeSmall = box
+                    break
 
-        # output all params into a star file
-        tableCryolo = Table(columns=['rlnParticleDiameter',
-                                     'rlnOriginalImageSize',
-                                     'rlnImageSize'])
-        tableCryolo.addRow(diam, boxSize, boxSizeSmall)
-        with open(outputFn, "w") as f:
-            tableCryolo.writeStar(f, tableName='picker')
+            print("\nSuggested parameters:\n\tDiameter (A): %d\n\tBox size (px): %d\n"
+                  "\tBox size binned (px): %d" % (diam, boxSize, boxSizeSmall))
+
+            # output all params into a star file
+            tableCryolo = Table(columns=['rlnParticleDiameter',
+                                         'rlnOriginalImageSize',
+                                         'rlnImageSize'])
+            tableCryolo.addRow(diam, boxSize, boxSizeSmall)
+            with open(outputFn, "w") as f:
+                tableCryolo.writeStar(f, tableName='picker')
 
         # create .gui_manualpickjob.star for easy display
         starString = """
 # version 30001
+
 data_job
-_rlnJobType                             3
+
+_rlnJobTypeLabel             relion.manualpick%s
 _rlnJobIsContinue                       0
+_rlnJobIsTomo                           0
+
 # version 30001
+
 data_joboptions_values
+
 loop_
 _rlnJobOptionVariable #1
 _rlnJobOptionValue #2
@@ -286,6 +267,7 @@ blue_value          0
 color_label rlnParticleSelectZScore
   diameter         %d
   do_color         No
+do_fom_threshold         No
   do_queue         No
 do_startend        No
   fn_color         ""
@@ -294,6 +276,7 @@ do_startend        No
    lowpass         20
   micscale        0.2
 min_dedicated       1
+minimum_pick_fom          0
 other_args         ""
       qsub       qsub
 qsubscript /public/EM/RELION/relion/bin/relion_qsub.csh
@@ -302,12 +285,9 @@ qsubscript /public/EM/RELION/relion/bin/relion_qsub.csh
 sigma_contrast      3
  white_val          0
 """
-        with open(getPath(".gui_manualpickjob.star"), "w") as f:
-            f.write(starString % (angpix, diam))
-
-    # remove output dir
-    if os.path.isdir("output"):
-        shutil.rmtree("output")
+        label = ".helical" if filament else ""
+        with open(".gui_manualpickjob.star", "w") as f:
+            f.write(starString % (label, angpix, diam))
 
     end = time.time()
     diff = end - start
@@ -317,7 +297,7 @@ sigma_contrast      3
 def main():
     """Change to the job working directory, then call run_job()"""
     help = """
-External job for calling cryolo within Relion 4.0. Run it in the Relion project directory, e.g.:
+External job for calling crYOLO 1.8+ within Relion 4.0. Run it in the Relion project directory, e.g.:
     external_job_cryolo.py --o External/cryolo_picking --in_mics CtfFind/job004/micrographs_ctf.star
 """
     parser = argparse.ArgumentParser(usage=help)
@@ -328,7 +308,6 @@ External job for calling cryolo within Relion 4.0. Run it in the Relion project
     parser.add_argument("--threshold", help="Threshold for picking (default = 0.3)", type=float, default=0.3)
     parser.add_argument("--model", help="Cryolo training model (if not specified general model is used)", default="None")
     parser.add_argument("--filament", help='Enable filament mode', default=False, action='store_true')
-    parser.add_argument("--fw", dest="filament_width", help='[FILAMENT MODE] Filament width (in pixel)', type=int, default=None)
     parser.add_argument("--bd", dest="box_distance", help='[FILAMENT MODE] Distance in pixel between two boxes', type=int, default=None)
     parser.add_argument("--mn", dest="minimum_number_boxes", help='[FILAMENT MODE] Minimum number of boxes per filament', type=int, default=None)
     parser.add_argument("--denoise", help="Denoise with JANNI instead of lowpass filtering", default=False, action='store_true')
@@ -346,19 +325,20 @@ External job for calling cryolo within Relion 4.0. Run it in the Relion project
         exit(1)
 
     if args.filament:
-        if None in [args.filament_width, args.box_distance, args.minimum_number_boxes]:
-            print("Error: --fw, --bd, --mn are required in filament mode!")
+        if None in [args.box_size, args.box_distance, args.minimum_number_boxes]:
+            print("Error: --box_size, --bd, --mn are required in filament mode!")
             exit(1)
 
-    project_dir = os.getcwd()
     os.makedirs(args.out_dir, exist_ok=True)
-    os.chdir(args.out_dir)
+    RELION_JOB_FAILURE_FILENAME = os.path.join(args.out_dir, "RELION_JOB_EXIT_FAILURE")
+    RELION_JOB_SUCCESS_FILENAME = os.path.join(args.out_dir, "RELION_JOB_EXIT_SUCCESS")
+
     if os.path.isfile(RELION_JOB_FAILURE_FILENAME):
         os.remove(RELION_JOB_FAILURE_FILENAME)
     if os.path.isfile(RELION_JOB_SUCCESS_FILENAME):
         os.remove(RELION_JOB_SUCCESS_FILENAME)
     try:
-        run_job(project_dir, args)
+        run_job(args)
     except:
         open(RELION_JOB_FAILURE_FILENAME, "w").close()
         raise
